from scipy.stats import kstest
from util import read_usertask, read_userinfo, load_answers, obtain_user_characteristic
from util import calc_system_advice, calc_numeracy, calc_TiA_scale, calc_ATI_scale, TiA_subscales, calc_reliance_measures
from util import task_list_1, task_list_2, task_list_3
# Load data from different csv files
usertask_dict, user_attention_check_correct, valid_user_set= read_usertask()
userinfo_dict = read_userinfo()
answer_dict = load_answers()
user_familiarity_dict, user_trust_dict, user_confidence_dict = obtain_user_characteristic()
user_TiA_scale = calc_TiA_scale()
user_ATI_scale = calc_ATI_scale()
user_numeracy_dict = calc_numeracy()

system_advice_dict, user_order_dict = calc_system_advice(userinfo_dict, answer_dict)
user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = \
	calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_order_dict)

def get_measures_analogy(user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict):
	analogy_res_dict = {}
	for var_name in ["appropriate_reliance", "accuracy", "agreement_fraction", "switching_fraction"]:
		analogy_res_dict[var_name] = {}
		for analogy_domain in ["train", "weather", "vaccine"]:
			analogy_res_dict[var_name][analogy_domain] = []
	for user in user_appropriate_reliance:
		for analogy_domain in ["train", "weather", "vaccine"]:
			analogy_res_dict["appropriate_reliance"][analogy_domain].append(user_appropriate_reliance[user][analogy_domain])
			analogy_res_dict["accuracy"][analogy_domain].append(accuracy_dict[user][analogy_domain])
			analogy_res_dict["agreement_fraction"][analogy_domain].append(user_agreement_fraction_dict[user][analogy_domain])
			analogy_res_dict["switching_fraction"][analogy_domain].append(user_switching_fraction_dict[user][analogy_domain])
	return analogy_res_dict

analogy_res_dict = get_measures_analogy(user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict)
for var_name in ["appropriate_reliance", "accuracy", "agreement_fraction", "switching_fraction"]:
	tp_list = []
	for analogy_domain in ["train", "weather", "vaccine"]:
		# print(analogy_res_dict[var_name][analogy_domain])
		# ks_statistic, p_value = kstest(analogy_res_dict[var_name][analogy_domain], 'norm')
		# print(var_name, analogy_domain)
		# print("ks_statistic: {:.2f}, p value: {:.2f}".format(ks_statistic, p_value))
		tp_list += analogy_res_dict[var_name][analogy_domain]
	print(var_name)
	ks_statistic, p_value = kstest(tp_list, 'norm')
	print("ks_statistic: {:.2f}, p value: {:.2f}".format(ks_statistic, p_value))
	if p_value < 0.05:
		print("Variable {} is not a normal distribution".format(var_name))
	print("-"*17)
