from scipy.stats import spearmanr
from util import read_usertask, read_userinfo, load_answers, obtain_user_characteristic
from util import calc_system_advice, calc_numeracy, calc_TiA_scale, calc_ATI_scale, TiA_subscales, calc_reliance_measures, calc_reliance_measures_2
from util import task_list_1, task_list_2, task_list_3
# Load data from different csv files
usertask_dict, user_attention_check_correct, valid_user_set= read_usertask()
userinfo_dict = read_userinfo()
answer_dict = load_answers()
user_familiarity_dict, user_trust_dict, user_confidence_dict = obtain_user_characteristic()
user_TiA_scale = calc_TiA_scale()
user_ATI_scale = calc_ATI_scale()
user_numeracy_dict = calc_numeracy()
system_advice_dict, user_order_dict = calc_system_advice(userinfo_dict, answer_dict)
user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = \
	calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_order_dict)

RAIR_dict, RSR_dict = calc_reliance_measures_2(usertask_dict, system_advice_dict, answer_dict, user_order_dict)


def test_spearman(cov_dict, dv_dict, analogy_domain, cov_name, dv_name, print_flag=True):
	cov_list = []
	dv_list = []
	for user in cov_dict:
		if analogy_domain == "all":
			for analogy_domain_ in ["train", "weather", "vaccine"]:
				cov_list.append(cov_dict[user][analogy_domain_])
				dv_list.append(dv_dict[user][analogy_domain_])
		else:
			assert analogy_domain_ in ["train", "weather", "vaccine"]
			cov_list.append(cov_dict[user][analogy_domain])
			dv_list.append(dv_dict[user][analogy_domain])
	assert len(cov_list) == len(dv_list)
	correlation, pvalue = spearmanr(cov_list, dv_list)
	# print(correlation, pvalue)
	print("On {} domain".format(analogy_domain))
	print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(cov_name, dv_name, correlation, pvalue))
	if print_flag:
		print("-" * 17)


def test_spearman_other(cov_dict, dv_dict, analogy_domain, cov_name, dv_name, print_flag=True):
	cov_list = []
	dv_list = []
	for user in cov_dict:
		if cov_name == "TiA-Propensity":
			tp_cov_val = cov_dict[user]["Propensity to Trust"]
		elif cov_name == "TiA-Familiarity":
			tp_cov_val = cov_dict[user]["Familiarity"]
		elif cov_name == "Numeracy_level":
			tp_cov_val = cov_dict[user][0]
		else:
			tp_cov_val = cov_dict[user]
		if analogy_domain == "all":
			# include all measures on three analogy domains for analysis
			for analogy_domain_ in ["train", "weather", "vaccine"]:
				cov_list.append(tp_cov_val)
				# For this case, we repeat it 3 times on 3 analogy domains
				dv_list.append(dv_dict[user][analogy_domain_])
		else:
			# include all measures on single analogy domain for analysis
			assert analogy_domain in ["train", "weather", "vaccine"]
			cov_list.append(tp_cov_val)
			dv_list.append(dv_dict[user][analogy_domain])
	assert len(cov_list) == len(dv_list)
	correlation, pvalue = spearmanr(cov_list, dv_list)
	# print(correlation, pvalue)
	print("On {} domain".format(analogy_domain))
	print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(cov_name, dv_name, correlation, pvalue))
	if print_flag:
		print("-" * 17)

# def obtain_cov_dict():
# 	# analogy domain specific factors
# 	user_familiarity_dict, user_trust_dict, user_confidence_dict = obtain_user_characteristic()
	
# 	# user specific
# 	user_TiA_scale = calc_TiA_scale()
# 	user_ATI_scale = calc_ATI_scale()
# 	user_numeracy_dict = calc_numeracy()
# 	cov_dict = {}
# 	tp_cov_name_dict = {
# 		"Trust": user_trust_dict,
# 		"Familiarity": user_familiarity_dict,
# 		"Confidence": user_confidence_dict,
# 		"TiA-Propensity": user_TiA_scale["Propensity to Trust"],
# 		"TiA-Familiarity": user_TiA_scale["Familiarity"],
# 	}
# 	user_order = list(user_familiarity_dict.keys())
# 	for var_name in ["Trust", "Familiarity", "Confidence", "TiA-Propensity", "TiA-Familiarity"]:
# 		cov_dict[var_name] = []
# 		for user in user_order:
# 			for analogy_domain_ in ["train", "weather", "vaccine"]:
# 				cov_dict["Trust"].append(user_trust_dict[user][analogy_domain_])
# 				cov_dict["Familiarity"].append(user_familiarity_dict[user][analogy_domain_])
# 				cov_dict["Confidence"].append(user_confidence_dict[user][analogy_domain_])
# 			cov_dict["TiA-Propensity"].append(user_TiA_scale[user]["Propensity to Trust"])
# 			cov_dict["TiA-Familiarity"].append(user_TiA_scale[user]["Familiarity"])


print("Results for Spearman Test")
print("-" * 34)

# test_spearman(user_trust_dict, user_appropriate_reliance, "all", "Trust", "appropriate_reliance")
# test_spearman(user_trust_dict, accuracy_dict, "all", "Trust", "accuracy")
# test_spearman(user_trust_dict, user_agreement_fraction_dict, "all", "Trust", "agreement_fraction")
# test_spearman(user_trust_dict, user_switching_fraction_dict, "all", "Trust", "switching_fraction")
# test_spearman(user_trust_dict, RAIR_dict, "all", "Trust", "RAIR")
# test_spearman(user_trust_dict, RSR_dict, "all", "Trust", "RSR", print_flag=False)

# print("-" * 34)

# test_spearman(user_familiarity_dict, user_appropriate_reliance, "all", "Familiarity", "appropriate_reliance")
# test_spearman(user_familiarity_dict, accuracy_dict, "all", "Familiarity", "accuracy")
# test_spearman(user_familiarity_dict, user_agreement_fraction_dict, "all", "Familiarity", "agreement_fraction")
# test_spearman(user_familiarity_dict, user_switching_fraction_dict, "all", "Familiarity", "switching_fraction")
# test_spearman(user_familiarity_dict, RAIR_dict, "all", "Familiarity", "RAIR")
# test_spearman(user_familiarity_dict, RSR_dict, "all", "Familiarity", "RSR", print_flag=False)

# print("-" * 34)

# test_spearman(user_confidence_dict, user_appropriate_reliance, "all", "Confidence", "appropriate_reliance")
# test_spearman(user_confidence_dict, accuracy_dict, "all", "Confidence", "accuracy")
# test_spearman(user_confidence_dict, user_agreement_fraction_dict, "all", "Confidence", "agreement_fraction")
# test_spearman(user_confidence_dict, user_switching_fraction_dict, "all", "Confidence", "switching_fraction")
# test_spearman(user_confidence_dict, RAIR_dict, "all", "Confidence", "RAIR")
# test_spearman(user_confidence_dict, RSR_dict, "all", "Confidence", "RSR", print_flag=False)

# print("-" * 34)
test_spearman_other(user_numeracy_dict, user_agreement_fraction_dict, "all", "Numeracy_level", "agreement_fraction")
test_spearman_other(user_numeracy_dict, user_switching_fraction_dict, "all", "Numeracy_level", "switching_fraction")
test_spearman_other(user_numeracy_dict, user_appropriate_reliance, "all", "Numeracy_level", "appropriate_reliance")
test_spearman_other(user_numeracy_dict, accuracy_dict, "all", "Numeracy_level", "accuracy")
test_spearman_other(user_numeracy_dict, RAIR_dict, "all", "Numeracy_level", "RAIR")
test_spearman_other(user_numeracy_dict, RSR_dict, "all", "Numeracy_level", "RSR", print_flag=False)

print("-" * 34)
test_spearman_other(user_ATI_scale, user_agreement_fraction_dict, "all", "ATI", "agreement_fraction")
test_spearman_other(user_ATI_scale, user_switching_fraction_dict, "all", "ATI", "switching_fraction")
test_spearman_other(user_ATI_scale, user_appropriate_reliance, "all", "ATI", "appropriate_reliance")
test_spearman_other(user_ATI_scale, accuracy_dict, "all", "ATI", "accuracy")
test_spearman_other(user_ATI_scale, RAIR_dict, "all", "ATI", "RAIR")
test_spearman_other(user_ATI_scale, RSR_dict, "all", "ATI", "RSR", print_flag=False)

print("-" * 34)
test_spearman_other(user_TiA_scale, user_agreement_fraction_dict, "all", "TiA-Familiarity", "agreement_fraction")
test_spearman_other(user_TiA_scale, user_switching_fraction_dict, "all", "TiA-Familiarity", "switching_fraction")
test_spearman_other(user_TiA_scale, user_appropriate_reliance, "all", "TiA-Familiarity", "appropriate_reliance")
test_spearman_other(user_TiA_scale, accuracy_dict, "all", "TiA-Familiarity", "accuracy")
test_spearman_other(user_TiA_scale, RAIR_dict, "all", "TiA-Familiarity", "RAIR")
test_spearman_other(user_TiA_scale, RSR_dict, "all", "TiA-Familiarity", "RSR", print_flag=False)

print("-" * 34)
test_spearman_other(user_TiA_scale, user_agreement_fraction_dict, "all", "TiA-Propensity", "agreement_fraction")
test_spearman_other(user_TiA_scale, user_switching_fraction_dict, "all", "TiA-Propensity", "switching_fraction")
test_spearman_other(user_TiA_scale, user_appropriate_reliance, "all", "TiA-Propensity", "appropriate_reliance")
test_spearman_other(user_TiA_scale, accuracy_dict, "all", "TiA-Propensity", "accuracy")
test_spearman_other(user_TiA_scale, RAIR_dict, "all", "TiA-Propensity", "RAIR")
test_spearman_other(user_TiA_scale, RSR_dict, "all", "TiA-Propensity", "RSR", print_flag=False)

# for analogy_domain in ["train", "weather", "vaccine"]:
# 	test_spearman(user_trust_dict, user_appropriate_reliance, analogy_domain, "Trust", "appropriate_reliance")
# 	test_spearman(user_trust_dict, accuracy_dict, analogy_domain, "Trust", "accuracy")
# 	test_spearman(user_trust_dict, user_agreement_fraction_dict, analogy_domain, "Trust", "agreement_fraction")
# 	test_spearman(user_trust_dict, user_switching_fraction_dict, analogy_domain, "Trust", "switching_fraction")
# 	test_spearman(user_trust_dict, RAIR_dict, analogy_domain, "Trust", "RAIR")
# 	test_spearman(user_trust_dict, RSR_dict, analogy_domain, "Trust", "RSR")
# test_spearman(user_familiarity_dict, user_appropriate_reliance, analogy_domain, "Familiarity", "appropriate_reliance")
# test_spearman(user_confidence_dict, user_appropriate_reliance, analogy_domain, "Confidence", "appropriate_reliance")

