import numpy as np
import pandas as pd
import os


data_folder = "../anonymous_data"
User_groups = ["system", "accuracy", "analogy"]
Analogy_types = ["train", "weather", "vaccine"]
vaccine_analogy = "The system is <strong>75%</strong> accurate, which is as reliable as the AstraZeneca vaccine to protect against covid"
train_analogy = "The system is <strong>75%</strong> accurate, which is about as reliable as the French trains are on punctuality"
weather_analogy = "the system is <strong>75%</strong> accurate, which is about as accurate as the seven day weather forecast"
# task_ID_list = ['LP001030', 'LP001806', 'LP002534', 'LP001882', 'LP002068', 'LP001849', 'LP002142', 'LP001451', 'LP002181', 'LP002840']
task_list_1 = ['LP002571', 'LP001633', 'LP001882', 'LP002181']
task_list_2 = ['LP001469', 'LP002500', 'LP002449', 'LP001722']
task_list_3 = ['LP002534', 'LP001451', 'LP001536', 'LP001996']

def reverse_answer(answer_str):
	if answer_str == "reject":
		return "accept"
	elif answer_str == "accept":
		return "reject"
	raise NotImplementedError("No such answer: {}".format(answer_str))

def deal_task_order_str(task_order_str):
	task_order_str_, analogy_order_str = task_order_str.strip().split(" ")
	task_order_list = task_order_str_.split("|")
	analogy_order_list = analogy_order_str.split("|")
	assert len(task_order_list) == 12
	assert len(analogy_order_list) == 3
	return task_order_list, analogy_order_list

def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_wrong_advice = {}
	user_order_dict = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		# taks_order_list = task_order_str.split("|")
		task_order_list, analogy_order_list = deal_task_order_str(task_order_str)
		user_order_dict[user] = (task_order_list, analogy_order_list)
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(task_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_order_dict

def load_answers():
	reserved_features = ['Loan_ID', 'Loan_Status', 'Sample_Category']
	answer_file = "../boarderline_12_tasks.csv"
	answer_df = pd.read_csv(answer_file, usecols=reserved_features)
	answer_list = answer_df.values.tolist()
	answer_dict = {}
	for Loan_ID, Loan_Status, Sample_Category in answer_list:
		tp_answer = "accept" if Loan_Status == "Y" else "reject"
		answer_dict[Loan_ID] = tp_answer
	return answer_dict

def collect_file_users(filename):
	df = pd.read_csv(filename, usecols=['user_id'])
	user_list = df.values.tolist()
	user_set = set([item[0] for item in user_list])
	return user_set

def check_completeness(usertask_dict, userinfo_dict):
	filenames = ["ATI.csv", "TiA_postq.csv", "pre_questionnaire.csv", "AnalogyPreQ.csv"]
	users_occ_list = []
	for filename in filenames:
		users_occ_list.append(collect_file_users(os.path.join(data_folder, filename)))
	valid_user_set = set()
	for user in usertask_dict:
		if len(usertask_dict[user]) < 24 or user not in userinfo_dict:
			continue
		user_id, x, xx, user_group, analogy_type = userinfo_dict[user]
		flag = False
		for tp_index in range(len(filenames)):
			if user not in users_occ_list[tp_index]:
				flag = True
				break
		if flag:
			continue
		valid_user_set.add(user)
	# print("Check Completeness Done.")
	return valid_user_set

def read_userinfo(reserved_users=None):
	filename = os.path.join(data_folder, "userinfo.csv")
	usecols = ['user_id','task_order_str','reverse_flag_str','user_group','analogy_type']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_status_list = df.values.tolist()
	userinfo_dict = {}
	for tuple_ in user_status_list:
		userinfo_dict[tuple_[0]] = tuple_
	# check data completeness for userinfo records
	if reserved_users is not None:
		for user_id in reserved_users:
			assert user_id in userinfo_dict
	# print("UserInfo Read Done.")
	return userinfo_dict

def calc_mean(value_list):
	return np.mean(value_list)

def reverse_code(value, max_scale):
	return max_scale + 1 - value

class questionnaire(object):
	
	def __init__(self, values, reverse_code_index, max_scale=6, questionnaire_size=0):
		self.questionnaire_size = questionnaire_size
		assert len(values) == self.questionnaire_size
		tp_values = []
		for i, value in enumerate(values):
			if (i + 1) in reverse_code_index:
				tp_values.append(reverse_code(value, max_scale))
			else:
				tp_values.append(value)
		self.values = tp_values
		self.max_scale = max_scale

	def calc_value(self):
		return calc_mean(self.values)

TiA_subscales = ["Familiarity", "Propensity to Trust"]

class TiA_questionnaire(object):
	
	def __init__(self, values):
		self.questionnaire_size = 5
		self.max_scale = 5
		self.reverse_code_index = [3]
		self.subscales = {
			"Familiarity": [1, 2],
			"Propensity to Trust": [3, 4, 5],
		}
		assert len(values) == self.questionnaire_size
		tp_values = []
		for i, value in enumerate(values):
			assert value >= 1 and value <= self.max_scale
			if (i + 1) in self.reverse_code_index:
				tp_values.append(reverse_code(value, self.max_scale))
			else:
				tp_values.append(value)
		self.values = tp_values
		self.scale_dict = {}

	def calc_value(self):
		for subscale in self.subscales:
			self.scale_dict[subscale] = calc_mean([self.values[index - 1] for index in self.subscales[subscale]])
			# print(subscale, self.scale_dict[subscale])
		# print("-" * 20)
		return self.scale_dict

def calc_numeracy(reserved_users=None):
	filename = os.path.join(data_folder, "pre_questionnaire.csv")
	df = pd.read_csv(filename)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_numeracy_list = df.values.tolist()
	user_numeracy_dict = {}
	reverse_code_index = [7]
	max_scale = 6
	for tuple_ in user_numeracy_list:
		user_id = tuple_[0]
		# answer_7 = 7 - answer_7
		# reverse code for answer_7
		# SNS_ability_sum = answer_1 + answer_2 + answer_3 + answer_4
		# SNS_preference_sum = answer_5 + answer_6 + answer_7 + answer_8
		# SNS = (SNS_ability_sum + SNS_preference_sum) / 8.0
		# SNS_ability = SNS_ability_sum / 4.0
		# SNS_preference = SNS_preference_sum / 4.0
		SNS = questionnaire(tuple_[1:-1], reverse_code_index=[7], max_scale=max_scale, questionnaire_size=8).calc_value()
		SNS_ability = questionnaire(tuple_[1:5], reverse_code_index=[], max_scale=max_scale, questionnaire_size=4).calc_value()
		SNS_preference = questionnaire(tuple_[5:9], reverse_code_index=[3], max_scale=max_scale, questionnaire_size=4).calc_value()
		user_numeracy_dict[user_id] = (SNS, SNS_ability, SNS_preference)
		# print(user_id, SNS, SNS_ability, SNS_preference)
	# print("Numeracy level Read Done.")
	if reserved_users is not None:
		for user_id in reserved_users:
			assert user_id in user_numeracy_dict
	return user_numeracy_dict

def calc_ATI_scale(reserved_users=None):
	filename = os.path.join(data_folder, "ATI.csv")
	df = pd.read_csv(filename)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_task_list = df.values.tolist()
	user_ATI_scale = {}
	reverse_code_index = [3, 6, 8]
	max_scale = 6
	questionnaire_size = 9
	# reverse code for question 3, 6, 8
	for tuple_ in user_task_list:
		# user_id, answer_1, answer_2, answer_3, answer_4, answer_5, answer_6, answer_7, answer_8, answer_9, __ = tuple_
		# reverse code for question 3, 6, 8
		# answer_3 = 7 - answer_3
		# answer_6 = 7 - answer_6
		# answer_8 = 7 - answer_8
		# ATI_scale = (answer_1 + answer_2 + answer_3 + answer_4 + answer_5 + answer_6 + answer_7 + answer_8 + answer_9) / 9.0
		ATI_scale = questionnaire(tuple_[1:-1], reverse_code_index, max_scale=max_scale, questionnaire_size=questionnaire_size).calc_value()
		user_id = tuple_[0]
		user_ATI_scale[user_id] = ATI_scale
	# print("ATI Read Done.")
	return user_ATI_scale

def obtain_user_characteristic(reserved_users=None):
	filename = os.path.join(data_folder, "AnalogyPreQ.csv")
	usecols = ["user_id"]
	for i in range(9):
		usecols.append("answer_{}".format(i+1))
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_familiarity_dict, user_trust_dict, user_confidence_dict = {}, {}, {}
	analogy_feedback_records = df.to_dict('records')
	# list of dict, every dict represents data for one user
	base_index = [1, 4, 7]
	shift = {"train": 0, "weather": 1, "vaccine": 2}
	for record in analogy_feedback_records:
		user_id = record['user_id']
		user_familiarity_dict[user_id] = {}
		user_trust_dict[user_id] = {}
		user_confidence_dict[user_id] = {}
		for i in range(9):
			tp_data = int(record['answer_{}'.format(i+1)])
			assert tp_data >= 1 and tp_data <= 6
		for analogy_domain in shift:
			tp_shift = shift[analogy_domain]
			user_familiarity_dict[user_id][analogy_domain] = int(record['answer_{}'.format(base_index[0] + tp_shift)])
			user_trust_dict[user_id][analogy_domain] = int(record['answer_{}'.format(base_index[1] + tp_shift)])
			user_confidence_dict[user_id][analogy_domain] = int(record['answer_{}'.format(base_index[2] + tp_shift)])
	# print("User Characteristic Read Done.")
	# check data completeness for usertask records
	if reserved_users is not None:
		for user_id in reserved_users:
			assert user_id in user_familiarity_dict
			assert user_id in user_trust_dict
			assert user_id in user_confidence_dict
			for analogy_domain in ["train", "weather", "vaccine"]:
				assert analogy_domain in user_familiarity_dict[user_id]
				assert analogy_domain in user_trust_dict[user_id]
				assert analogy_domain in user_confidence_dict[user_id]
	return user_familiarity_dict, user_trust_dict, user_confidence_dict

def calc_TiA_scale(reserved_users=None):
	filename = os.path.join(data_folder, "TiA_postq.csv")
	usecols = ['user_id']
	for i in range(5):
		usecols.append("answer_{}".format(i+1))
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_task_list = df.values.tolist()
	user_TiA_scale = {}
	for tuple_ in user_task_list:
		TiA_scale = TiA_questionnaire(tuple_[1:]).calc_value()
		user_id = tuple_[0]
		user_TiA_scale[user_id] = TiA_scale
	# print("TiA Read Done.")
	# check data completeness for usertask records
	if reserved_users is not None:
		for user_id in reserved_users:
			assert user_id in user_TiA_scale
	return user_TiA_scale


def read_usertask(reserved_users=None):
	filename = os.path.join(data_folder, "usertask.csv")
	usecols = ['user_id', 'task_id', 'answer_type', 'choice']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_task_list = df.values.tolist()

	user_task_dict = {}
	user_attention_check_correct = {}
	for user_id, task_id, answer_type, choice in user_task_list:
		if user_id not in user_task_dict:
			user_task_dict[user_id] = {}
		if user_id not in user_attention_check_correct:
			user_attention_check_correct[user_id] = 0
		if answer_type == "attention":
			if task_id == "LP001518" and choice == "yes":
				user_attention_check_correct[user_id] += 1
			if task_id == "LP002723" and choice == "no":
				user_attention_check_correct[user_id] += 1
			if task_id == "ATI_attention" and choice == "6":
				user_attention_check_correct[user_id] += 1
			if task_id == "Numeracy Test" and choice == "3":
				user_attention_check_correct[user_id] += 1
			if task_id == "TiA_attention" and choice == "1":
				user_attention_check_correct[user_id] += 1
		else:
			assert choice in ['accept', 'reject']
			user_task_dict[user_id][(task_id, answer_type)] = choice
	# check data completeness for usertask records
	valid_user_set = set()
	if reserved_users is not None:
		for user_id in reserved_users:
			user_lack_records = []
			for task_id in task_list_1 + task_list_2 + task_list_3:
				if (task_id, "loan_analogy") not in user_task_dict[user_id]:
					user_lack_records.append((task_id, "loan_analogy"))
				if (task_id, "base") not in user_task_dict[user_id]:
					user_lack_records.append((task_id, "base"))
				# assert (task_id, "loan_analogy") in user_task_dict[user_id]
				# assert (task_id, "base") in user_task_dict[user_id]
			if len(user_lack_records) == 0:
				valid_user_set.add(user_id)
			else:
				print("Incomplete user: {}".format(user_id))
				print("Lack records: ", user_lack_records)
	# print("UserTask Read Done.")
	return user_task_dict, user_attention_check_correct, valid_user_set

def load_demographic(filename="prolific_demographic_round1.csv"):
	filename = os.path.join(data_folder, filename)
	df = pd.read_csv(filename, usecols=['participant_id'])
	user_list = df.values.tolist()
	user_set = set([item[0] for item in user_list])
	return user_set

def calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_order_dict):
	accuracy_dict = {}
	user_appropriate_reliance = {}
	user_agreement_fraction_dict = {}
	user_switching_fraction_dict = {}
	number_no_disagreement = 0
	for user in usertask_dict:
		task_order_list, analogy_order_list = user_order_dict[user]
		accuracy_dict[user] = {}
		user_appropriate_reliance[user] = {}
		user_agreement_fraction_dict[user] = {}
		user_switching_fraction_dict[user] = {}
		for shift, analogy_domain in enumerate(analogy_order_list):
			tp_correct = 0
			tp_agreement = 0
			initial_disagreement = 0
			tp_switch_reliance = 0
			tp_correct_initial_disagreement = 0
			for i in range(4):
				task_index = i + shift * 4
				task_id = task_order_list[task_index]
				first_choice = usertask_dict[user][(task_id, "base")]
				second_choice = usertask_dict[user][(task_id, "loan_analogy")]
				system_advice = system_advice_dict[user][task_id]
				correct_answer = answer_dict[task_id]
				if second_choice == system_advice:
					# agreement fraction
					tp_agreement += 1
				if first_choice != system_advice:
					# initial disagreement
					initial_disagreement += 1
					if second_choice == system_advice:
						tp_switch_reliance += 1
					if second_choice == correct_answer:
						tp_correct_initial_disagreement += 1
				if second_choice == correct_answer:
					tp_correct += 1
			accuracy_dict[user][analogy_domain] = tp_correct / 4.0
			user_agreement_fraction_dict[user][analogy_domain] = tp_agreement / 4.0
			if initial_disagreement > 0:
				user_switching_fraction_dict[user][analogy_domain] = float(tp_switch_reliance) / initial_disagreement
				user_appropriate_reliance[user][analogy_domain] = float(tp_correct_initial_disagreement) / initial_disagreement
			else:
				number_no_disagreement += 1
				user_switching_fraction_dict[user][analogy_domain] = 0.0
				user_appropriate_reliance[user][analogy_domain] = 0.0
	print("There are {} user freq agree with system advice all the time on one domain.".format(number_no_disagreement))
	return user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict


# https://chi-trait.github.io/papers/CHI_TRAIT_2022_Paper_55.pdf
def calc_reliance_measures_2(usertask_dict, system_advice_dict, answer_dict, user_order_dict):
	number_no_disagreement = 0
	RAIR_dict = {}
	# relative positive AI reliance
	RSR_dict = {}
	# relative positive self-reliance
	for user in usertask_dict:
		task_order_list, analogy_order_list = user_order_dict[user]
		RAIR_dict[user] = {}
		RSR_dict[user] = {}
		for shift, analogy_domain in enumerate(analogy_order_list):
			initial_disagreement = 0
			Positive_AI_reliance = 0
			Negative_AI_reliance = 0
			Negative_self_reliance = 0
			Positive_self_reliance = 0
			for i in range(4):
				task_index = i + shift * 4
				task_id = task_order_list[task_index]
				first_choice = usertask_dict[user][(task_id, "base")]
				second_choice = usertask_dict[user][(task_id, "loan_analogy")]
				system_advice = system_advice_dict[user][task_id]
				correct_answer = answer_dict[task_id]
				if first_choice != system_advice:
					# initial disagreement
					initial_disagreement += 1
					if system_advice == correct_answer:
						if second_choice == correct_answer:
							Positive_AI_reliance += 1
						else:
							Negative_self_reliance += 1
					else:
						if second_choice == correct_answer:
							Positive_self_reliance += 1
						else:
							Negative_AI_reliance += 1
			if Positive_AI_reliance + Negative_AI_reliance > 0:
				RAIR_dict[user][analogy_domain] = float(Positive_AI_reliance) / (Positive_AI_reliance + Negative_AI_reliance)
			else:
				RAIR_dict[user][analogy_domain] = 0.0
			if Positive_self_reliance + Negative_self_reliance > 0:
				RSR_dict[user][analogy_domain] = float(Positive_self_reliance) / (Positive_self_reliance + Negative_self_reliance)
			else:
				RSR_dict[user][analogy_domain] = 0.0
			if initial_disagreement == 0:
				number_no_disagreement += 1
	print("There are {} user freq agree with system advice all the time on one domain.".format(number_no_disagreement))
	return RAIR_dict, RSR_dict

if __name__ == "__main__":
	# filter_invalid_users()
	# calc_ATI_scale()
	round1_users = load_demographic()
	userinfo_dict = read_userinfo(reserved_users=round1_users)
	usertask_dict, user_attention_check_correct, valid_user_set = read_usertask(reserved_users=round1_users)
	user_TiA_scale = calc_TiA_scale(reserved_users=round1_users)
	user_familiarity_dict, user_trust_dict, user_confidence_dict = obtain_user_characteristic(reserved_users=round1_users)
	user_numeracy_dict = calc_numeracy(reserved_users=round1_users)
	# valid_user_set = check_completeness(usertask_dict=usertask_dict, userinfo_dict=userinfo_dict)
	# print(len(valid_user_set), len(all_user_set), len(valid_user_set & all_user_set))












