import numpy as np
import pandas as pd

from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import reverse_answer, obtain_analogy_familiarity, calc_numeracy, calc_reliance_measures
from util import case_dict, trust_dict, task_ID_list


def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

def test_kurskal(data_list_1, data_list_2, data_list_3, name):
	# Compute the Kruskal-Wallis H-test for independent samples.
	from scipy import stats
	statistic, pvalue = stats.kruskal(data_list_1, data_list_2, data_list_3)
	print("kruskal test result: H:{:.3f}, p:{:.3f}".format(statistic, pvalue))
	print("Mean for system: {:.3f}, accuracy: {:.3f}, analogy: {:.3f}".format(np.mean(data_list_1), np.mean(data_list_2), np.mean(data_list_3)))
	print("SD for system: {:.3f}, accuracy: {:.3f}, analogy: {:.3f}".format(np.std(data_list_1, ddof=1), np.std(data_list_2, ddof=1), np.std(data_list_3, ddof=1)))
	# np.std(loan_data_variables[key_], ddof=1)
	if pvalue >= 0.0125:
		print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, name))
	else:
		print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, name))
		# statistic, pvalue = stats.mannwhitneyu(data_list_1, data_list_2, alternative='two-sided')
		# print("system", "accuracy", "%.4f"%pvalue, "%.4f"%statistic)
		# statistic, pvalue = stats.mannwhitneyu(data_list_1, data_list_3, alternative='two-sided')
		# print("system", "analogy", "%.4f"%pvalue, "%.4f"%statistic)
		# statistic, pvalue = stats.mannwhitneyu(data_list_2, data_list_1, alternative='two-sided')
		# print("accuracy", "analogy", "%.4f"%pvalue, "%.4f"%statistic)
		statistic, pvalue = stats.mannwhitneyu(data_list_1, data_list_2, alternative='two-sided')
		print("Alternative system <> accuracy,", "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		# statistic, pvalue = stats.mannwhitneyu(data_list_1, data_list_2, alternative='greater')
		# print("Alternative system > accuracy,", "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		# statistic, pvalue = stats.mannwhitneyu(data_list_1, data_list_3, alternative='greater')
		# print("Alternative system > analogy,", "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		# statistic, pvalue = stats.mannwhitneyu(data_list_2, data_list_3, alternative='greater')
		# print("Alternative accuracy > analogy,", "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)


if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, calc_ATI_scale, TiA_subscales
	user_TiA_scale = calc_TiA_scale()
	user_ATI_scale = calc_ATI_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	user_numeracy_dict = calc_numeracy()

	user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	loan_data_variables = {
		"user_id": [],
		"user_group": [],
		"analogy_type": [],
		"Numeracy_level": [],
		"Analogy_domain_familiarity": [],
		"ATI": [],
		"Appropriate_reliance": [],
		"Accuracy": [],
		"Agreement_fraction": [],
		"Switching_fraction": [],
		"userfulness": []
	}
	for subscale in TiA_subscales:
		loan_data_variables["TiA-{}".format(subscale)] = []
	for user in reserved_users:
		loan_data_variables["user_id"].append(user)
		# if user_group_dict[user] == "analogy":
		# 	loan_data_variables["user_group"].append(user_group_dict[user] + "-" + user_analogy_dict[user])
		# else:
		loan_data_variables["user_group"].append(user_group_dict[user])
		loan_data_variables["Numeracy_level"].append(user_numeracy_dict[user][0])

		if user_group_dict[user] != "analogy":
			loan_data_variables["analogy_type"].append(None)
			loan_data_variables["Analogy_domain_familiarity"].append(None)
			loan_data_variables["userfulness"].append(None)
		else:
			loan_data_variables["analogy_type"].append(user_analogy_dict[user])
			loan_data_variables["Analogy_domain_familiarity"].append(user_familiarity_dict[user])
			loan_data_variables["userfulness"].append(user_usefulness_dict[user])
		loan_data_variables["ATI"].append(user_ATI_scale[user])
		
		loan_data_variables["Appropriate_reliance"].append(user_appropriate_reliance[user])
		loan_data_variables["Accuracy"].append(accuracy_dict[user])
		loan_data_variables["Agreement_fraction"].append(user_agreement_fraction_dict[user])
		loan_data_variables["Switching_fraction"].append(user_switching_fraction_dict[user])
		for subscale in TiA_subscales:
			loan_data_variables["TiA-{}".format(subscale)].append(user_TiA_scale[user][subscale])
	for key_ in loan_data_variables:
		print(key_, len(loan_data_variables[key_]))
	# for key_ in ["ATI", "Numeracy_level", "TiA-Propensity to Trust", "TiA-Familiarity", "Accuracy"]:
	# 	print(key_)
	# 	print("mean:{:.2f} std:{:.2f}".format(np.mean(loan_data_variables[key_]), np.std(loan_data_variables[key_], ddof=1)))
	df = pd.DataFrame(loan_data_variables)
	from pingouin import ancova, pairwise_tukey, kruskal, mwu

	print("-" * 19)

	### result for Sec 5.2.1
	print("For dependent variable Appropriate_reliance")
	# print(kruskal(data=df, dv='Appropriate_reliance', between='user_group'))
	data_list = {}
	for user in reserved_users:
		tp_group = user_group_dict[user]
		if tp_group not in data_list:
			data_list[tp_group] = []
		data_list[tp_group].append(user_appropriate_reliance[user])
	test_kurskal(data_list["system"], data_list["accuracy"], data_list["analogy"], "appropriate reliance")
	
	print("For dependent variable Accuracy")
	# print(kruskal(data=df, dv='Accuracy', between='user_group'))
	data_list = {}
	for user in reserved_users:
		tp_group = user_group_dict[user]
		if tp_group not in data_list:
			data_list[tp_group] = []
		data_list[tp_group].append(accuracy_dict[user])
	test_kurskal(data_list["system"], data_list["accuracy"], data_list["analogy"], "participant accuracy")

	print("For dependent variable Agreement_fraction")
	# print(kruskal(data=df, dv='Trust_behavior', between='user_group'))
	data_list = {}
	for user in reserved_users:
		tp_group = user_group_dict[user]
		if tp_group not in data_list:
			data_list[tp_group] = []
		data_list[tp_group].append(user_agreement_fraction_dict[user])
	test_kurskal(data_list["system"], data_list["accuracy"], data_list["analogy"], "Agreement_fraction")

	print("For dependent variable Switching_fraction")
	# print(kruskal(data=df, dv='Trust_behavior', between='user_group'))
	data_list = {}
	for user in reserved_users:
		tp_group = user_group_dict[user]
		if tp_group not in data_list:
			data_list[tp_group] = []
		data_list[tp_group].append(user_switching_fraction_dict[user])
	test_kurskal(data_list["system"], data_list["accuracy"], data_list["analogy"], "Switching_fraction")

	print("-" * 19)

	### result for first paragraph of Table 3
	print("For independent variable TiA-Reliability/Competence")
	print(ancova(data=df, dv='TiA-Reliability/Competence', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust'], between='user_group', effsize='n2'))
	
	print("For independent variable TiA-Understanding/Predictability")
	print(ancova(data=df, dv='TiA-Understanding/Predictability', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust'], between='user_group', effsize='n2'))
	
	print("For independent variable TiA-Intention of Developers")
	print(ancova(data=df, dv='TiA-Intention of Developers', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust'], between='user_group', effsize='n2'))

	print("For independent variable TiA-Trust in Automation")
	print(ancova(data=df, dv='TiA-Trust in Automation', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust'], between='user_group', effsize='n2'))
	
	
	





