import numpy as np
import pandas as pd
import os


from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import User_groups, Analogy_types, task_ID_list, obtain_analogy_familiarity, calc_numeracy, calc_reliance_measures
from util import case_dict, trust_dict, reverse_answer

def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

# used only given 

def calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	switching_cases = {}
	accuracy_dict = {}
	user_defend = {}
	user_appropriate_reliance = {}
	user_trust_behavior_dict = {}
	user_reliance_list = []
	user_trust_behavior_list = []
	user_correct_list = []
	switching_analogy_cases = {}
	for user in usertask_dict:
		user_group = user_group_dict[user]
		analogy_type = user_analogy_dict[user]
		user_trust_list = []
		tp_correct = 0
		appropriate_reliance = 0
		defend_wrong = 0
		tp_trust = 0
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			case = case_dict[(first_choice, second_choice, system_advice)]
			trust_pattern = trust_dict[case]
			if trust_pattern in ["weak trust", "strong trust"]:
				tp_trust += 1
			if system_advice != correct_answer:
				# system provide a wrong answer
				if trust_pattern in ["weak distrust"]:
					# participants defend wrong system advice by insisting their first decision
					appropriate_reliance += 1
					defend_wrong += 1
			else:
				# system provide a correct answer
				if trust_pattern in ["weak trust", "strong trust"]:
					# participants adopt correct system advice by switching to system's 
					# or agree with system's advice
					appropriate_reliance += 1
			if second_choice == correct_answer:
				tp_correct += 1
			trust_pattern = trust_dict[case]
			user_trust_list.append(trust_pattern)
		user_appropriate_reliance[user] = appropriate_reliance
		# user_defend[user] = defend_wrong / 3.0
		# switching_cases[user].append(user_trust_list)
		accuracy_dict[user] = tp_correct / 10.0
		user_trust_behavior_dict[user] = tp_trust
	return user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict

def test_spearman(variable_1_list, variable_2_list, name1, name2):
	from scipy import stats
	# alternative{‘two-sided’, ‘less’, ‘greater’}, optional
	# Defines the alternative hypothesis. Default is ‘two-sided’. The following options are available:
	# 	‘two-sided’: the correlation is nonzero
	# 	‘less’: the correlation is negative (less than zero)
	# 	‘greater’: the correlation is positive (greater than zero)
	correlation, pvalue = stats.spearmanr(variable_1_list, variable_2_list)
	# print(correlation, pvalue)
	print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(name1, name2, correlation, pvalue))

if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, TiA_subscales, calc_ATI_scale
	user_TiA_scale = calc_TiA_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	user_numeracy_dict = calc_numeracy()
	user_ATI_scale = calc_ATI_scale()

	# user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict = calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	# familiarty_list = []

	### result for first paragraph of Table 9
	SNS_list = []
	appropriate_reliance_list = []
	accuracy_list = []
	trust_behavior_list = []
	agreement_fraction_list = []
	switching_fraction_list = []
	Propensity_list = []
	ATI_list = []
	Familiarity_list = []
	# print(len(user_appropriate_reliance))
	print("-" * 17)
	print("For all participants")
	for user in user_numeracy_dict:
		# analogy group
		if user not in reserved_users:
			continue
		SNS, __, ___ = user_numeracy_dict[user]
		SNS_list.append(SNS)
		ATI_list.append(user_ATI_scale[user])
		Propensity_list.append(user_TiA_scale[user]["Propensity to Trust"])
		Familiarity_list.append(user_TiA_scale[user]["Familiarity"])
		appropriate_reliance_list.append(user_appropriate_reliance[user])
		accuracy_list.append(accuracy_dict[user])
		# trust_behavior_list.append(user_trust_behavior_dict[user])
		agreement_fraction_list.append(user_agreement_fraction_dict[user])
		switching_fraction_list.append(user_switching_fraction_dict[user])
	print("{} participants are taken into consideration".format(len(SNS_list)))
	test_spearman(SNS_list, appropriate_reliance_list, "SNS", "appropriate_reliance")
	test_spearman(SNS_list, accuracy_list, "SNS", "accuracy")
	test_spearman(SNS_list, agreement_fraction_list, "SNS", "Agreement_fraction")
	test_spearman(SNS_list, switching_fraction_list, "SNS", "Switching fraction")
	# test_spearman(SNS_list, trust_behavior_list, "SNS", "strong trust behavior")

	test_spearman(ATI_list, appropriate_reliance_list, "ATI", "appropriate_reliance")
	test_spearman(ATI_list, accuracy_list, "ATI", "accuracy")
	# test_spearman(ATI_list, trust_behavior_list, "ATI", "strong trust behavior")
	test_spearman(ATI_list, agreement_fraction_list, "ATI", "Agreement_fraction")
	test_spearman(ATI_list, switching_fraction_list, "ATI", "Switching fraction")

	test_spearman(Propensity_list, appropriate_reliance_list, "Propensity to Trust", "appropriate_reliance")
	test_spearman(Propensity_list, accuracy_list, "Propensity to Trust", "accuracy")
	# test_spearman(Propensity_list, trust_behavior_list, "Propensity to Trust", "strong trust behavior")
	test_spearman(Propensity_list, agreement_fraction_list, "Propensity to Trust", "Agreement_fraction")
	test_spearman(Propensity_list, switching_fraction_list, "Propensity to Trust", "Switching fraction")

	test_spearman(Familiarity_list, appropriate_reliance_list, "Familiarity", "appropriate_reliance")
	test_spearman(Familiarity_list, accuracy_list, "Familiarity", "accuracy")
	# test_spearman(Familiarity_list, trust_behavior_list, "Familiarity", "strong trust behavior")
	test_spearman(Familiarity_list, agreement_fraction_list, "Familiarity", "Agreement_fraction")
	test_spearman(Familiarity_list, switching_fraction_list, "Familiarity", "Switching fraction")
	print("-" * 34)

	### result for trust-related measures
	TiA_subscales_ = ["Reliability/Competence", "Understanding/Predictability", "Intention of Developers", "Trust in Automation"]
	subscale_list = {}
	user_list_subscale_dict = {}
	for subscale in TiA_subscales_:
		user_list_subscale_dict[subscale] = []
	SNS_list = []
	Propensity_list = []
	ATI_list = []
	Familiarity_list = []
	for user in user_numeracy_dict:
		if user not in reserved_users:
			continue
		SNS, __, ___ = user_numeracy_dict[user]
		SNS_list.append(SNS)
		ATI_list.append(user_ATI_scale[user])
		Familiarity_list.append(user_TiA_scale[user]["Familiarity"])
		Propensity_list.append(user_TiA_scale[user]["Propensity to Trust"])
		for subscale in TiA_subscales_:
			user_list_subscale_dict[subscale].append(user_TiA_scale[user][subscale])
	print("-" * 17)
	print("For TiA subscales")
	print("All {} participants are taken into consideration".format(len(SNS_list)))
	# for subscale in TiA_subscales_:
	# 	test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)
	print("-" * 17)
	for subscale in TiA_subscales_:
		test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)
		test_spearman(ATI_list, user_list_subscale_dict[subscale], "ATI", subscale)
		test_spearman(Propensity_list, user_list_subscale_dict[subscale], "Propensity to Trust", subscale)
		test_spearman(Familiarity_list, user_list_subscale_dict[subscale], "Familiarity", subscale)





