import numpy as np
import pandas as pd
import os


from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import User_groups, Analogy_types, task_ID_list, obtain_analogy_familiarity, calc_numeracy, calc_reliance_measures
from util import case_dict, trust_dict, reverse_answer

def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

def test_spearman(variable_1_list, variable_2_list, name1, name2):
	from scipy import stats
	# alternative{‘two-sided’, ‘less’, ‘greater’}, optional
	# Defines the alternative hypothesis. Default is ‘two-sided’. The following options are available:
	# 	‘two-sided’: the correlation is nonzero
	# 	‘less’: the correlation is negative (less than zero)
	# 	‘greater’: the correlation is positive (greater than zero)
	correlation, pvalue = stats.spearmanr(variable_1_list, variable_2_list)
	# print(correlation, pvalue)
	print("Variable {} and variable {} have spearman correlation {:.3f} and pvalue {:.3f}".format(name1, name2, correlation, pvalue))

if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, TiA_subscales, calc_ATI_scale
	user_TiA_scale = calc_TiA_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	user_numeracy_dict = calc_numeracy()
	user_ATI_scale = calc_ATI_scale()

	user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

	# SNS_list = []
	# appropriate_reliance_list = []
	# accuracy_list = []
	# Propensity_list = []
	# ATI_list = []
	# Familiarity_list = []
	# # print(len(user_appropriate_reliance))
	# print("For all participants")
	# print("-" * 34)
	# for user in user_numeracy_dict:
	# 	# analogy group
	# 	if user not in reserved_users:
	# 		continue
	# 	SNS, __, ___ = user_numeracy_dict[user]
	# 	SNS_list.append(SNS)
	# 	ATI_list.append(user_ATI_scale[user])
	# 	Propensity_list.append(user_TiA_scale[user]["Propensity to Trust"])
	# 	Familiarity_list.append(user_TiA_scale[user]["Familiarity"])
	# 	appropriate_reliance_list.append(user_appropriate_reliance[user])
	# 	accuracy_list.append(accuracy_dict[user])
	# print("{} participants are taken into consideration".format(len(SNS_list)))
	# test_spearman(SNS_list, appropriate_reliance_list, "SNS", "appropriate_reliance")
	# test_spearman(SNS_list, accuracy_list, "SNS", "accuracy")
	print("-" * 34)
	# test_spearman(Propensity_list, SNS_list, "SNS", "Propensity to Trust")

	### results for table 4
	# code to calculate the Spearman's rank correlation coefficient for SNS and appropriate_reliance, accuracy, agreement fraction, switching fraction
	# for condition in ["system", "accuracy", "analogy"]:
	for condition in ["analogy"]:
		SNS_list = []
		appropriate_reliance_list = []
		accuracy_list = []
		# trust_behavior_list = []
		user_agreement_fraction_list = []
		user_switching_fraction_list = []
		Propensity_list = []
		ATI_list = []
		# print(len(user_appropriate_reliance))
		print("-" * 17)
		print("For group {}".format(condition))
		for user in user_numeracy_dict:
			# analogy group
			if user not in reserved_users:
				continue
			if user_group_dict[user] != condition:
				continue
			SNS, __, ___ = user_numeracy_dict[user]
			SNS_list.append(SNS)
			# ATI_list.append(user_ATI_scale[user])
			# Propensity_list.append(user_TiA_scale[user]["Propensity to Trust"])
			appropriate_reliance_list.append(user_appropriate_reliance[user])
			accuracy_list.append(accuracy_dict[user])
			# trust_behavior_list.append(user_trust_behavior_dict[user])
			user_agreement_fraction_list.append(user_agreement_fraction_dict[user])
			user_switching_fraction_list.append(user_switching_fraction_dict[user])
		print("{} participants are taken into consideration".format(len(SNS_list)))
		test_spearman(SNS_list, user_agreement_fraction_list, "SNS", "Agreement_fraction")
		test_spearman(SNS_list, user_switching_fraction_list, "SNS", "Switching_fraction")
		test_spearman(SNS_list, appropriate_reliance_list, "SNS", "appropriate_reliance")
		test_spearman(SNS_list, accuracy_list, "SNS", "accuracy")
		# test_spearman(Propensity_list, appropriate_reliance_list, "Propensity to Trust", "appropriate_reliance")
		# test_spearman(Propensity_list, accuracy_list, "Propensity to Trust", "accuracy")
		# test_spearman(ATI_list, appropriate_reliance_list, "ATI", "appropriate_reliance")
		# test_spearman(ATI_list, accuracy_list, "ATI", "accuracy")

	for condition in ["train", "weather", "vaccine"]:
		SNS_list = []
		appropriate_reliance_list = []
		accuracy_list = []
		user_agreement_fraction_list = []
		user_switching_fraction_list = []
		# print(len(user_appropriate_reliance))
		print("-" * 17)
		print("For group analogy-{}".format(condition))
		for user in user_numeracy_dict:
			# analogy group
			if user not in reserved_users:
				continue
			if user_group_dict[user] != "analogy" or user_analogy_dict[user] != condition:
				continue
			SNS, __, ___ = user_numeracy_dict[user]
			SNS_list.append(SNS)
			appropriate_reliance_list.append(user_appropriate_reliance[user])
			accuracy_list.append(accuracy_dict[user])
			user_agreement_fraction_list.append(user_agreement_fraction_dict[user])
			user_switching_fraction_list.append(user_switching_fraction_dict[user])
		print("{} participants are taken into consideration".format(len(SNS_list)))
		test_spearman(SNS_list, user_agreement_fraction_list, "SNS", "Agreement_fraction")
		test_spearman(SNS_list, user_switching_fraction_list, "SNS", "Switching_fraction")
		test_spearman(SNS_list, appropriate_reliance_list, "SNS", "appropriate_reliance")
		test_spearman(SNS_list, accuracy_list, "SNS", "accuracy")
	# variable_1_list = range(1, 100)
	# variable_2_list = [item * -8 + 3 for item in variable_1_list]
	# test_spearman(variable_1_list, variable_2_list, "1 - 100", "x * 8 + 3")

	# ### result for first paragraph of Table 6, results for trust
	# TiA_subscales_ = ["Reliability/Competence", "Understanding/Predictability", "Intention of Developers", "Trust in Automation"]
	# subscale_list = {}
	# user_list_subscale_dict = {}
	# for subscale in TiA_subscales_:
	# 	user_list_subscale_dict[subscale] = []
	# SNS_list = []
	# Propensity_list = []
	# ATI_list = []
	# Familiarity_list = []
	# for user in user_numeracy_dict:
	# 	if user not in reserved_users:
	# 		continue
	# 	SNS, __, ___ = user_numeracy_dict[user]
	# 	SNS_list.append(SNS)
	# 	ATI_list.append(user_ATI_scale[user])
	# 	Familiarity_list.append(user_TiA_scale[user]["Familiarity"])
	# 	Propensity_list.append(user_TiA_scale[user]["Propensity to Trust"])
	# 	for subscale in TiA_subscales_:
	# 		user_list_subscale_dict[subscale].append(user_TiA_scale[user][subscale])
	# print("-" * 17)
	# print("For TiA subscales")
	# print("All {} participants are taken into consideration".format(len(SNS_list)))
	# # for subscale in TiA_subscales_:
	# # 	test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)
	# print("-" * 17)
	# for subscale in TiA_subscales_:
	# 	test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)
	# 	# test_spearman(ATI_list, user_list_subscale_dict[subscale], "ATI", subscale)
	# 	# test_spearman(Propensity_list, user_list_subscale_dict[subscale], "Propensity to Trust", subscale)
	# 	# test_spearman(Familiarity_list, user_list_subscale_dict[subscale], "Familiarity", subscale)
	# # import sys
	# # sys.exit(-1)

	# for condition in ["system", "accuracy", "analogy"]:
	# 	subscale_list = {}
	# 	user_list_subscale_dict = {}
	# 	for subscale in TiA_subscales_:
	# 		user_list_subscale_dict[subscale] = []
	# 	SNS_list = []
	# 	for user in user_numeracy_dict:
	# 		if user not in reserved_users:
	# 			continue
	# 		if user_group_dict[user] != condition:
	# 			continue
	# 		SNS, __, ___ = user_numeracy_dict[user]
	# 		SNS_list.append(SNS)
	# 		for subscale in TiA_subscales_:
	# 			user_list_subscale_dict[subscale].append(user_TiA_scale[user][subscale])
	# 	print("-" * 17)
	# 	print("{} group have {} participants are taken into consideration".format(condition, len(SNS_list)))
	# 	for subscale in TiA_subscales_:
	# 		test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)

	# for condition in ["train", "weather", "vaccine"]:
	# 	subscale_list = {}
	# 	user_list_subscale_dict = {}
	# 	for subscale in TiA_subscales_:
	# 		user_list_subscale_dict[subscale] = []
	# 	SNS_list = []
	# 	for user in user_numeracy_dict:
	# 		if user not in reserved_users:
	# 			continue
	# 		if user_group_dict[user] != "analogy" or user_analogy_dict[user] != condition:
	# 			continue
	# 		SNS, __, ___ = user_numeracy_dict[user]
	# 		SNS_list.append(SNS)
	# 		for subscale in TiA_subscales_:
	# 			user_list_subscale_dict[subscale].append(user_TiA_scale[user][subscale])
	# 	print("-" * 17)
	# 	print("Analogy-{} group have {} participants are taken into consideration".format(condition, len(SNS_list)))
	# 	for subscale in TiA_subscales_:
	# 		test_spearman(SNS_list, user_list_subscale_dict[subscale], "SNS", subscale)




