import numpy as np
import pandas as pd

from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import User_groups, Analogy_types, task_ID_list, obtain_analogy_familiarity, calc_numeracy, calc_reliance_measures
from util import case_dict, trust_dict, reverse_answer


def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

def show_pivot(user_numeracy_dict, reserved_users):
	tp_list = []
	for user_id in user_numeracy_dict:
		if user_id not in reserved_users:
			continue
		SNS, SNS_ability, SNS_preference = user_numeracy_dict[user_id]
		tp_list.append((user_id, SNS))
	tp_list.sort(key=lambda x:x[1], reverse=True)
	length = len(tp_list)
	pivot_index_1 = int(length / 4)
	pivot_index_2 = int(length / 4 * 3)
	# print(np.mean(tp_list))
	# print("Max:{:.2f}, pivot 1: {:.2f}, pivot 2: {:.2f} Min: {:.2f}".format(tp_list[0][1], tp_list[pivot_index_1][1], tp_list[pivot_index_2][1], tp_list[-1][1]))
	# pivot_value_1 = tp_list[pivot_index_1][1]
	# pivot_value_2 = tp_list[pivot_index_2][1]
	user_group_1 = set()
	user_group_2 = set()
	user_group_3 = set()
	for index, (user_id, SNS) in enumerate(tp_list):
		if index < pivot_index_1:
			user_group_1.add(user_id)
		elif index < pivot_index_2:
			user_group_2.add(user_id)
		else:
			user_group_3.add(user_id)
		# if SNS > pivot_value_1:
		# 	user_group_1.add(user_id)
		# elif SNS > pivot_value_2:
		# 	user_group_2.add(user_id)
		# else:
		# 	user_group_3.add(user_id)
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_1), tp_list[pivot_index_1][1], tp_list[0][1]))
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_2), tp_list[pivot_index_2][1], tp_list[pivot_index_1][1]))
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_3), tp_list[-1][1], tp_list[pivot_index_2][1]))
	print("{} participants has high numeracy level".format(len(user_group_1)))
	print("{} participants has medium numeracy level".format(len(user_group_2)))
	print("{} participants has low numeracy level".format(len(user_group_3)))
	return user_group_1, user_group_2, user_group_3


if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, calc_ATI_scale, TiA_subscales
	user_TiA_scale = calc_TiA_scale()
	user_ATI_scale = calc_ATI_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	reserved_users_new = reserved_users
	# for user in reserved_users:
	# 	if user_group_dict[user] == "analogy":
	# 		reserved_users_new.add(user)
	user_numeracy_dict = calc_numeracy(reserved_users_new)

	high_SNS_group, medium_SNS_group, low_SNS_group = show_pivot(user_numeracy_dict, reserved_users_new)

	user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

	dependent_variables_dict = {
		"Agreement_fraction": user_agreement_fraction_dict,
		"Switching_fraction": user_switching_fraction_dict,
		"Appropriate_reliance": user_appropriate_reliance,
		"Accuracy": accuracy_dict,
	}
	name_map = {
		"Agreement_fraction": "agreement fraction",
		"Switching_fraction": "switching fraction",
		"Appropriate_reliance": "appropriate reliance",
		"Accuracy": "participant accuracy"
	}
	data_variables = {}
	from scipy import stats
	# data_va
	### results for table 5
	for dv in dependent_variables_dict:
		data_dict = dependent_variables_dict[dv]
		high_list, medium_list, low_list = [], [], []
		for user in high_SNS_group:
			high_list.append(data_dict[user])
		for user in medium_SNS_group:
			medium_list.append(data_dict[user])
		for user in low_SNS_group:
			low_list.append(data_dict[user])
		print("For dependent variable {}".format(dv))
		print("Mean for high: {:.2f}, medium: {:.2f}, low: {:.2f}".format(np.mean(high_list), np.mean(medium_list), np.mean(low_list)))
		# data_variables[name_map[dv]] = [np.mean(high_list), np.mean(medium_list), np.mean(low_list)]
		statistic, pvalue = stats.kruskal(high_list, medium_list, low_list)
		print("statistic", statistic)
		if pvalue >= 0.0125:
			print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, name_map[dv]))
		else:
			print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, name_map[dv]))
		# 	# print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, medium_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: medium")
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, low_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: low")
		# 	statistic, pvalue = stats.mannwhitneyu(medium_list, low_list, alternative='two-sided')
		# 	print("alternative less: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("medium: low")
	for subscale in TiA_subscales:
		if subscale in ["Propensity to Trust", "Familiarity"]:
			continue
		dv = "TiA-{}".format(subscale)
		high_list, medium_list, low_list = [], [], []
		for user in high_SNS_group:
			high_list.append(user_TiA_scale[user][subscale])
		for user in medium_SNS_group:
			medium_list.append(user_TiA_scale[user][subscale])
		for user in low_SNS_group:
			low_list.append(user_TiA_scale[user][subscale])
		print("For dependent variable {}".format(dv))
		print("Mean for high: {:.2f}, medium: {:.2f}, low: {:.2f}".format(np.mean(high_list), np.mean(medium_list), np.mean(low_list)))
		# data_variables[name_map[dv]] = [np.mean(high_list), np.mean(medium_list), np.mean(low_list)]
		statistic, pvalue = stats.kruskal(high_list, medium_list, low_list)
		print("statistic", statistic)
		if pvalue >= 0.0125:
			print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, dv))
		else:
			print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, dv))
		# 	# print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, medium_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: medium")
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, low_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: low")
		# 	statistic, pvalue = stats.mannwhitneyu(medium_list, low_list, alternative='two-sided')
		# 	print("alternative less: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("medium: low")





