import numpy as np
import pandas as pd
import os


from util import filter_invalid_users, read_usertask, read_userinfo, load_answers, calc_reliance_measures
from util import User_groups, Analogy_types, task_ID_list
from util import case_dict, trust_dict, reverse_answer
from scipy import stats

data_folder = "../anonymous_data"
def reserve_user_feedback(reserved_users):
	filename = os.path.join(data_folder, "PostQ_analogy.csv")
	df = pd.read_csv(filename, usecols=['user_id', "answer_1", "answer_2"])
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	analogy_feedback_list = df.values.tolist()
	familiar_group = set()
	unfamiliar_group = set()
	user_analogy_helpfulness = {}
	for user_id, usefulness, familiarity in analogy_feedback_list:
		user_analogy_helpfulness[user_id] = usefulness
		if familiarity > 3:
			familiar_group.add(user_id)
		else:
			unfamiliar_group.add(user_id)
	return familiar_group, unfamiliar_group, user_analogy_helpfulness

def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

def calc_analogy_domain_usefulness(reserved_users, user_analogy_helpfulness, user_group_dict, user_analogy_dict):
	user_helpfness_dict = {}
	for item in ["train", "weather", "vaccine"]:
		user_helpfness_dict[item] = []
	for user in reserved_users:
		if user_group_dict[user] != "analogy":
			continue
		tp_analogy = user_analogy_dict[user]
		user_helpfness_dict[tp_analogy].append(user_analogy_helpfulness[user])
	statistic, pvalue = stats.kruskal(user_helpfness_dict["train"], user_helpfness_dict["weather"], user_helpfness_dict["vaccine"])
	print(statistic)
	if pvalue >= 0.05:
		print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, "helpfulness"))
	else:
		print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, "helpfulness"))
		print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["train"], user_helpfness_dict["weather"], alternative='less')
		print("alternative train < weather, pvalue: %.4f"%pvalue, "%.4f"%statistic)
		# print("train: weather")
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["train"], user_helpfness_dict["vaccine"], alternative='less')
		print("alternative train < vaccine, pvalue:  %.4f"%pvalue, "%.4f"%statistic)
		# print("train: vaccine")
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["weather"], user_helpfness_dict["vaccine"], alternative='two-sided')
		print("alternative weather <> vaccine pvalue: %.4f"%pvalue, "%.4f"%statistic)
		# print("weather: vaccine")


reserved_users = filter_invalid_users(exclude_round1=False)
usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
userinfo_dict = read_userinfo(reserved_users)
familiar_group, unfamiliar_group, user_analogy_helpfulness = reserve_user_feedback(reserved_users)
answer_dict = load_answers()
system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)

from util import calc_TiA_scale, TiA_subscales, obtain_analogy_familiarity
user_TiA_scale = calc_TiA_scale()
user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()

user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

# calc_switching_familiarity(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict, familiar_group, unfamiliar_group, user_usefulness_dict)
# calc_analogy_domain_usefulness(reserved_users, user_analogy_helpfulness, user_group_dict, user_analogy_dict)
# import sys
# sys.exit(-1)
# from util import calc_TiA_scale, TiA_subscales, obtain_analogy_familiarity
# user_TiA_scale = calc_TiA_scale()
# user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
# user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict = calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

# code for verifing H4 on dependent variables appropriate_reliance, accuracy, user_trust_behavior
familiarty_appropriate_reliance_dict = {}
familiarty_accuracy_dict = {}
familiarty_agreement_fraction_dict = {}
familiarty_switching_fraction_dict = {}
familiarty_user_usefulness_dict = {}

for user in user_familiarity_dict:
	if user not in reserved_users:
		continue
	if user_group_dict[user] != "analogy":
		continue
	tp_familiarity = user_familiarity_dict[user]
	if tp_familiarity not in familiarty_appropriate_reliance_dict:
		familiarty_appropriate_reliance_dict[tp_familiarity] = []
	if tp_familiarity not in familiarty_accuracy_dict:
		familiarty_accuracy_dict[tp_familiarity] = []
	if tp_familiarity not in familiarty_agreement_fraction_dict:
		familiarty_agreement_fraction_dict[tp_familiarity] = []
	if tp_familiarity not in familiarty_switching_fraction_dict:
		familiarty_switching_fraction_dict[tp_familiarity] = []
	if tp_familiarity not in familiarty_user_usefulness_dict:
		familiarty_user_usefulness_dict[tp_familiarity] = []
	familiarty_appropriate_reliance_dict[tp_familiarity].append(user_appropriate_reliance[user])
	familiarty_accuracy_dict[tp_familiarity].append(accuracy_dict[user])
	familiarty_agreement_fraction_dict[tp_familiarity].append(user_agreement_fraction_dict[user])
	familiarty_switching_fraction_dict[tp_familiarity].append(user_switching_fraction_dict[user])
	familiarty_user_usefulness_dict[tp_familiarity].append(user_analogy_helpfulness[user])


dependent_variables = {
	"Agreement_fraction": familiarty_agreement_fraction_dict,
	"Switching_fraction": familiarty_switching_fraction_dict,
	"appropriate_reliance": familiarty_appropriate_reliance_dict,
	"accuracy": familiarty_accuracy_dict
	# "helpfuness": familiarty_user_usefulness_dict
}

for index in [1, 2, 3, 4, 5]:
	print("Faimilarity {} has {} participants".format(index, len(familiarty_appropriate_reliance_dict[index])))
print("-" * 17)
for name in dependent_variables:
	statistic, pvalue = stats.kruskal(dependent_variables[name][1], dependent_variables[name][2],\
	dependent_variables[name][3], dependent_variables[name][4],\
	dependent_variables[name][5])
	# print("statistic: {:.3f}".format(statistic))
	if pvalue >= 0.0125:
		print("With pvalue: {:.3f}, H: {:.3f}, the five groups have same population mean with dependent variable {}".format(pvalue, statistic, name))
	else:
		print("With pvalue: {:.3f}, H: {:.3f},  the five groups don't have same population mean with dependent variable {}, check it more".format(pvalue, statistic, name))
		for i in [1, 2, 3, 4, 5]:
			for j in [1, 2, 3, 4, 5]:
				if i>= j:
					continue
				statistic, pvalue = stats.mannwhitneyu(dependent_variables[name][i], dependent_variables[name][j], alternative='less')
				# print(i, j, "%.4f"%pvalue)
				print("Alternative {} < {}, with pvalue: {:.4f}".format(i, j, pvalue))
				print("Mean for {}: {:.2f}; Mean for{}: {:.2f}".format(i, np.mean(dependent_variables[name][i]), j, np.mean(dependent_variables[name][j])))

# code for for verifing H4 on dependent variables Trust in Automation
dependent_variables_2 = {}
for subscale in TiA_subscales:
	dependent_variables_2[subscale] = {}
	for label in [1, 2, 3, 4, 5]:
		dependent_variables_2[subscale][label] = []
for user in user_familiarity_dict:
	if user not in reserved_users:
		continue
	if user_group_dict[user] != "analogy":
		continue
	tp_familiarity = user_familiarity_dict[user]
	for subscale in TiA_subscales: 
		dependent_variables_2[subscale][tp_familiarity].append(user_TiA_scale[user][subscale])
for name in dependent_variables_2:
	statistic, pvalue = stats.kruskal(dependent_variables_2[name][1], dependent_variables_2[name][2],\
	dependent_variables_2[name][3], dependent_variables_2[name][4],\
	dependent_variables_2[name][5])
	print("For variable TiA-{}".format(name))
	print("Mean for 1: {:.2f}, 2: {:.2f}, 3: {:.2f}, 4: {:.2f}, 5: {:.2f}".format(np.mean(dependent_variables_2[name][1]), np.mean(dependent_variables_2[name][2]),\
	 np.mean(dependent_variables_2[name][3]), np.mean(dependent_variables_2[name][4]), np.mean(dependent_variables_2[name][5])))
	print("SD for 1: {:.3f}, 2: {:.3f}, 3: {:.3f}, 4: {:.3f}, 5: {:.3f}".format(np.std(dependent_variables_2[name][1], ddof=1), np.std(dependent_variables_2[name][2], ddof=1),\
	 np.std(dependent_variables_2[name][3], ddof=1), np.std(dependent_variables_2[name][4], ddof=1), np.std(dependent_variables_2[name][5], ddof=1)))
	# print("statistic: {:.3f}".format(statistic))
	if pvalue >= 0.0125:
		print("With pvalue: {:.3f}, H: {:.3f}, the five groups have same population mean with dependent variable {}".format(pvalue, statistic, name))
	else:
		print("With pvalue: {:.3f}, H: {:.3f}, the five groups don't have same population mean with dependent variable {}, check it more".format(pvalue, statistic, name))
		for i in [1, 2, 3, 4, 5]:
			for j in [1, 2, 3, 4, 5]:
				if i>= j:
					continue
				statistic, pvalue = stats.mannwhitneyu(dependent_variables_2[name][i], dependent_variables_2[name][j], alternative='less')
				print("Alternative {} < {}, with pvalue: {:.4f}".format(i, j, pvalue))
				# print("Mean for {}: {:.2f}; Mean for{}: {:.2f}".format(i, np.mean(dependent_variables_2[name][i]), j, np.mean(dependent_variables_2[name][j])))





