import numpy as np
import pandas as pd

from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import reverse_answer, obtain_analogy_familiarity, calc_numeracy
from util import case_dict, trust_dict, task_ID_list


def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

# used only given 
def calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	switching_cases = {}
	accuracy_dict = {}
	user_defend = {}
	user_appropriate_reliance = {}
	user_trust_behavior_dict = {}
	user_reliance_list = []
	user_trust_behavior_list = []
	user_correct_list = []
	switching_analogy_cases = {}
	for user in usertask_dict:
		user_group = user_group_dict[user]
		analogy_type = user_analogy_dict[user]
		user_trust_list = []
		tp_correct = 0
		appropriate_reliance = 0
		defend_wrong = 0
		tp_trust = 0
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			case = case_dict[(first_choice, second_choice, system_advice)]
			trust_pattern = trust_dict[case]
			if trust_pattern in ["weak trust", "strong trust"]:
				tp_trust += 1
			if system_advice != correct_answer:
				# system provide a wrong answer
				if trust_pattern in ["weak distrust"]:
					# participants defend wrong system advice by insisting their first decision
					appropriate_reliance += 1
					defend_wrong += 1
			else:
				# system provide a correct answer
				if trust_pattern in ["weak trust", "strong trust"]:
					# participants adopt correct system advice by switching to system's 
					# or agree with system's advice
					appropriate_reliance += 1
			if second_choice == correct_answer:
				tp_correct += 1
			trust_pattern = trust_dict[case]
			user_trust_list.append(trust_pattern)
		user_appropriate_reliance[user] = appropriate_reliance
		# user_defend[user] = defend_wrong / 3.0
		# switching_cases[user].append(user_trust_list)
		accuracy_dict[user] = tp_correct / 10.0
		user_trust_behavior_dict[user] = tp_trust
	return user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict

def calc_user_switch(usertask_dict, reserved_users, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	switched_users = 0
	switched_total = 0
	total_decision = 0
	for user in reserved_users:
		switch_flag = False
		user_group = user_group_dict[user]
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			case = case_dict[(first_choice, second_choice, system_advice)]
			trust_pattern = trust_dict[case]
			if first_choice != second_choice:
				switch_flag = True
				switched_total += 1
			total_decision += 1
		if switch_flag:
			switched_users += 1
	print("-" * 17)
	print("{} participants switched at least one decision, it occupies ratio: {:.3f}".format(switched_users, switched_users/ float(len(reserved_users))))
	print("Among {} decisions, {:.3f} ratio of it are not switched".format(total_decision, (total_decision - switched_total) / float(total_decision)))


# code to report table 2
def calc_task_acc(usertask_dict, reserved_users, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	task_acc = {}
	error_nonreliance = {}
	error_reliance = {}
	appropriate_reliance = {}
	print("-" * 17)
	for task_id in task_ID_list:
		task_acc[task_id] = 0.0
		error_nonreliance[task_id] = 0.0
		error_reliance[task_id] = 0.0
		initial_disagreement = 0.0
		correct_disagreement = 0.0
		for user in reserved_users:
			user_group = user_group_dict[user]
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			if first_choice != system_advice:
				initial_disagreement += 1.0
				if second_choice == correct_answer:
					correct_disagreement += 1.0
			if second_choice == correct_answer:
				task_acc[task_id] += 1.0
			else:
				# error cause analysis
				if second_choice == system_advice:
					# user rely on system advice
					error_reliance[task_id] += 1.0
				else:
					# user don't rely on system advice
					error_nonreliance[task_id] += 1.0
		tp_appropriate_reliance = 0.0 if initial_disagreement == 0.0 else correct_disagreement / initial_disagreement
		print("For task {}, accuracy: {:.3f}, appropriate_reliance: {:.3f}".format(task_id, task_acc[task_id]/ len(reserved_users), tp_appropriate_reliance))
		print("Correct answer: {}".format(answer_dict[task_id]))
		error_total = error_reliance[task_id] + error_nonreliance[task_id]
		print("Error caused by reliance: {:.3f}, non-reliance: {:.3f}".format(error_reliance[task_id] / error_total, error_nonreliance[task_id]/ error_total))
	error_10tasks_reliance = 0
	error_10tasks_nonreliance = 0
	for task_id in task_ID_list:
		error_10tasks_reliance += error_reliance[task_id]
		error_10tasks_nonreliance += error_nonreliance[task_id]
	print("In all 10 tasks,")
	print("Error caused by reliance: {:.3f}, non-reliance: {:.3f}".format(error_10tasks_reliance / (error_10tasks_reliance + error_10tasks_nonreliance),\
	 error_10tasks_nonreliance / (error_10tasks_reliance + error_10tasks_nonreliance)))

if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, calc_ATI_scale, TiA_subscales
	user_TiA_scale = calc_TiA_scale()
	user_ATI_scale = calc_ATI_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	user_numeracy_dict = calc_numeracy()

	user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict = calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	loan_data_variables = {
		"user_id": [],
		"user_group": [],
		"analogy_type": [],
		"Numeracy_level": [],
		"Analogy_domain_familiarity": [],
		"ATI": [],
		"Appropriate_reliance": [],
		"Accuracy": [],
		"Trust_behavior": [],
		"userfulness": []
	}
	for subscale in TiA_subscales:
		loan_data_variables["TiA-{}".format(subscale)] = []
	# result for 5.1
	for user in reserved_users:
		loan_data_variables["user_id"].append(user)
		# if user_group_dict[user] == "analogy":
		# 	loan_data_variables["user_group"].append(user_group_dict[user] + "-" + user_analogy_dict[user])
		# else:
		loan_data_variables["user_group"].append(user_group_dict[user])
		loan_data_variables["Numeracy_level"].append(user_numeracy_dict[user][0])

		if user_group_dict[user] != "analogy":
			loan_data_variables["analogy_type"].append(None)
			# loan_data_variables["Analogy_domain_familiarity"].append(None)
			loan_data_variables["userfulness"].append(None)
		else:
			loan_data_variables["analogy_type"].append(user_analogy_dict[user])
			loan_data_variables["Analogy_domain_familiarity"].append(user_familiarity_dict[user])
			loan_data_variables["userfulness"].append(user_usefulness_dict[user])
		loan_data_variables["ATI"].append(user_ATI_scale[user])
		loan_data_variables["Appropriate_reliance"].append(user_appropriate_reliance[user])
		loan_data_variables["Accuracy"].append(accuracy_dict[user])
		loan_data_variables["Trust_behavior"].append(user_trust_behavior_dict[user])
		for subscale in TiA_subscales:
			loan_data_variables["TiA-{}".format(subscale)].append(user_TiA_scale[user][subscale])
	for key_ in loan_data_variables:
		print(key_, len(loan_data_variables[key_]))
	for key_ in ["ATI", "Numeracy_level", "TiA-Propensity to Trust", "TiA-Familiarity", "Analogy_domain_familiarity", "Accuracy"]:
		print(key_)
		print("mean:{:.2f} std:{:.2f}".format(np.mean(loan_data_variables[key_]), np.std(loan_data_variables[key_], ddof=1)))
	# df = pd.DataFrame(loan_data_variables)
	calc_user_switch(usertask_dict, reserved_users, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	calc_task_acc(usertask_dict, reserved_users, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	





