import numpy as np
import pandas as pd

from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import User_groups, Analogy_types, task_ID_list, obtain_analogy_familiarity, calc_numeracy, calc_reliance_measures
from util import case_dict, trust_dict, reverse_answer


def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

# used only given 
def calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	switching_cases = {}
	accuracy_dict = {}
	user_defend = {}
	user_appropriate_reliance = {}
	user_trust_behavior_dict = {}
	user_reliance_list = []
	user_trust_behavior_list = []
	user_correct_list = []
	switching_analogy_cases = {}
	for user in usertask_dict:
		user_group = user_group_dict[user]
		analogy_type = user_analogy_dict[user]
		user_trust_list = []
		tp_correct = 0
		appropriate_reliance = 0
		defend_wrong = 0
		tp_trust = 0
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			case = case_dict[(first_choice, second_choice, system_advice)]
			trust_pattern = trust_dict[case]
			if trust_pattern in ["weak trust", "strong trust"]:
				tp_trust += 1
			if system_advice != correct_answer:
				# system provide a wrong answer
				if trust_pattern in ["weak distrust"]:
					# participants defend wrong system advice by insisting their first decision
					appropriate_reliance += 1
					defend_wrong += 1
			else:
				# system provide a correct answer
				if trust_pattern in ["weak trust", "strong trust"]:
					# participants adopt correct system advice by switching to system's 
					# or agree with system's advice
					appropriate_reliance += 1
			if second_choice == correct_answer:
				tp_correct += 1
			trust_pattern = trust_dict[case]
			user_trust_list.append(trust_pattern)
		user_appropriate_reliance[user] = appropriate_reliance
		# user_defend[user] = defend_wrong / 3.0
		# switching_cases[user].append(user_trust_list)
		accuracy_dict[user] = tp_correct / 10.0
		user_trust_behavior_dict[user] = tp_trust
	return user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict

def show_pivot(user_numeracy_dict, reserved_users):
	tp_list = []
	for user_id in user_numeracy_dict:
		if user_id not in reserved_users:
			continue
		SNS, SNS_ability, SNS_preference = user_numeracy_dict[user_id]
		tp_list.append((user_id, SNS))
	tp_list.sort(key=lambda x:x[1], reverse=True)
	length = len(tp_list)
	pivot_index_1 = int(length / 4)
	pivot_index_2 = int(length / 4 * 3)
	# print(np.mean(tp_list))
	# print("Max:{:.2f}, pivot 1: {:.2f}, pivot 2: {:.2f} Min: {:.2f}".format(tp_list[0][1], tp_list[pivot_index_1][1], tp_list[pivot_index_2][1], tp_list[-1][1]))
	# pivot_value_1 = tp_list[pivot_index_1][1]
	# pivot_value_2 = tp_list[pivot_index_2][1]
	user_group_1 = set()
	user_group_2 = set()
	user_group_3 = set()
	for index, (user_id, SNS) in enumerate(tp_list):
		if index < pivot_index_1:
			user_group_1.add(user_id)
		elif index < pivot_index_2:
			user_group_2.add(user_id)
		else:
			user_group_3.add(user_id)
		# if SNS > pivot_value_1:
		# 	user_group_1.add(user_id)
		# elif SNS > pivot_value_2:
		# 	user_group_2.add(user_id)
		# else:
		# 	user_group_3.add(user_id)
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_1), tp_list[pivot_index_1][1], tp_list[0][1]))
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_2), tp_list[pivot_index_2][1], tp_list[pivot_index_1][1]))
	# print("{} participants has SNS in ({:.2f}, {:.2f}]".format(len(user_group_3), tp_list[-1][1], tp_list[pivot_index_2][1]))
	print("{} participants has high numeracy level".format(len(user_group_1)))
	print("{} participants has medium numeracy level".format(len(user_group_2)))
	print("{} participants has low numeracy level".format(len(user_group_3)))
	return user_group_1, user_group_2, user_group_3


if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)
	from util import calc_TiA_scale, calc_ATI_scale, TiA_subscales
	user_TiA_scale = calc_TiA_scale()
	user_ATI_scale = calc_ATI_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	reserved_users_new = reserved_users
	# for user in reserved_users:
	# 	if user_group_dict[user] == "analogy":
	# 		reserved_users_new.add(user)
	user_numeracy_dict = calc_numeracy(reserved_users_new)

	high_SNS_group, medium_SNS_group, low_SNS_group = show_pivot(user_numeracy_dict, reserved_users_new)

	# user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict = calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)
	user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict = calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

	# loan_data_variables = {
	# 	"user_id": [],
	# 	"user_group": [],
	# 	"analogy_type": [],
	# 	"Numeracy_level": [],
	# 	"Analogy_domain_familiarity": [],
	# 	"ATI": [],
	# 	"Appropriate_reliance": [],
	# 	"Accuracy": [],
	# 	"Trust_behavior": [],
	# 	"userfulness": []
	# }
	# for subscale in TiA_subscales:
	# 	loan_data_variables["TiA-{}".format(subscale)] = []
	# for user in reserved_users:
	# 	# if user_group_dict[user] != "analogy":
	# 	# 	continue
	# 	loan_data_variables["user_id"].append(user)
	# 	# if user_group_dict[user] == "analogy":
	# 	# 	loan_data_variables["user_group"].append(user_group_dict[user] + "-" + user_analogy_dict[user])
	# 	# else:
	# 	loan_data_variables["user_group"].append(user_group_dict[user])
	# 	loan_data_variables["Numeracy_level"].append(user_numeracy_dict[user][0])

	# 	if user_group_dict[user] != "analogy":
	# 		loan_data_variables["analogy_type"].append(None)
	# 		loan_data_variables["Analogy_domain_familiarity"].append(None)
	# 		loan_data_variables["userfulness"].append(None)
	# 	else:
	# 		loan_data_variables["analogy_type"].append(user_analogy_dict[user])
	# 		loan_data_variables["Analogy_domain_familiarity"].append(user_familiarity_dict[user])
	# 		loan_data_variables["userfulness"].append(user_usefulness_dict[user])
	# 	loan_data_variables["ATI"].append(user_ATI_scale[user])
	# 	loan_data_variables["Appropriate_reliance"].append(user_appropriate_reliance[user])
	# 	loan_data_variables["Accuracy"].append(accuracy_dict[user])
	# 	loan_data_variables["Trust_behavior"].append(user_trust_behavior_dict[user])
	# 	for subscale in TiA_subscales:
	# 		loan_data_variables["TiA-{}".format(subscale)].append(user_TiA_scale[user][subscale])
	# for key_ in loan_data_variables:
	# 	print(key_, len(loan_data_variables[key_]))
	# for key_ in ["ATI", "Numeracy_level", "TiA-Propensity to Trust", "TiA-Familiarity", "Accuracy"]:
	# 	print(key_)
	# 	print("mean:{:.2f} std:{:.2f}".format(np.mean(loan_data_variables[key_]), np.std(loan_data_variables[key_], ddof=1)))
	# df = pd.DataFrame(loan_data_variables)
	dependent_variables_dict = {
		"Appropriate_reliance": user_appropriate_reliance,
		"Accuracy": accuracy_dict,
		"Agreement_fraction": user_agreement_fraction_dict,
		"Switching_fraction": user_switching_fraction_dict,
	}
	name_map = {
		"Appropriate_reliance": "appropriate reliance",
		"Accuracy": "participant accuracy",
		"Agreement_fraction": "agreement fraction",
		"Switching_fraction": "switching fraction"
	}
	data_variables = {}
	from scipy import stats
	# data_va
	### results for table 8
	for dv in dependent_variables_dict:
		data_dict = dependent_variables_dict[dv]
		high_list, medium_list, low_list = [], [], []
		for user in high_SNS_group:
			high_list.append(data_dict[user])
		for user in medium_SNS_group:
			medium_list.append(data_dict[user])
		for user in low_SNS_group:
			low_list.append(data_dict[user])
		# print("For dependent variable {}".format(dv))
		# print("Mean for high: {:.2f}, medium: {:.2f}, low: {:.2f}".format(np.mean(high_list), np.mean(medium_list), np.mean(low_list)))
		# data_variables[name_map[dv]] = [np.mean(high_list), np.mean(medium_list), np.mean(low_list)]
		statistic, pvalue = stats.kruskal(high_list, medium_list, low_list)
		print("statistic", statistic)
		if pvalue >= 0.0125:
			print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, name_map[dv]))
		else:
			print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, name_map[dv]))
		# 	# print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, medium_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: medium")
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, low_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: low")
		# 	statistic, pvalue = stats.mannwhitneyu(medium_list, low_list, alternative='two-sided')
		# 	print("alternative less: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("medium: low")
	for subscale in TiA_subscales:
		dv = "TiA-{}".format(subscale)
		high_list, medium_list, low_list = [], [], []
		for user in high_SNS_group:
			high_list.append(user_TiA_scale[user][subscale])
		for user in medium_SNS_group:
			medium_list.append(user_TiA_scale[user][subscale])
		for user in low_SNS_group:
			low_list.append(user_TiA_scale[user][subscale])
		# print("For dependent variable {}".format(dv))
		# print("Mean for high: {:.2f}, medium: {:.2f}, low: {:.2f}".format(np.mean(high_list), np.mean(medium_list), np.mean(low_list)))
		# data_variables[name_map[dv]] = [np.mean(high_list), np.mean(medium_list), np.mean(low_list)]
		statistic, pvalue = stats.kruskal(high_list, medium_list, low_list)
		print("statistic", statistic)
		if pvalue >= 0.0125:
			print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, dv))
		else:
			print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, dv))
		# 	# print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, medium_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: medium")
		# 	statistic, pvalue = stats.mannwhitneyu(high_list, low_list, alternative='two-sided')
		# 	print("alternative two-sided: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("high: low")
		# 	statistic, pvalue = stats.mannwhitneyu(medium_list, low_list, alternative='two-sided')
		# 	print("alternative less: %.4f"%pvalue, "%.4f"%statistic)
		# 	print("medium: low")





