import numpy as np
import pandas as pd
import os


data_folder = "../anonymous_data"
User_groups = ["system", "accuracy", "analogy"]
Analogy_types = ["train", "weather", "vaccine"]
vaccine_analogy = "The system is <strong>75%</strong> accurate, which is as reliable as the AstraZeneca vaccine to protect against covid"
train_analogy = "The system is <strong>75%</strong> accurate, which is about as reliable as the French trains are on punctuality"
weather_analogy = "the system is <strong>75%</strong> accurate, which is about as accurate as the seven day weather forecast"
task_ID_list = ['LP001030', 'LP001806', 'LP002534', 'LP001882', 'LP002068', 'LP001849', 'LP002142', 'LP001451', 'LP002181', 'LP002840']

case_dict = {
	('accept', 'accept', 'accept') : 1,
	('accept', 'accept', 'reject') : 2,
	('accept', 'reject', 'accept') : 3,
	('accept', 'reject', 'reject') : 4,
	('reject', 'accept', 'accept') : 5,
	('reject', 'accept', 'reject') : 6,
	('reject', 'reject', 'accept') : 7,
	('reject', 'reject', 'reject') : 8,
}

reliance_dict = {
	1: "reliance",
	2: "non-reliance",
	3: "non-reliance",
	4: "reliance",
	5: "reliance",
	6: "non-reliance",
	7: "non-reliance",
	8: "reliance"
}

trust_dict = {
	1: "weak trust",
	2: "weak distrust",
	3: "strong distrust",
	4: "strong trust",
	5: "strong trust",
	6: "strong distrust",
	7: "weak distrust",
	8: "weak trust"
}

def reverse_answer(answer_str):
	if answer_str == "reject":
		return "accept"
	elif answer_str == "accept":
		return "reject"
	raise NotImplementedError("No such answer: {}".format(answer_str))

def load_answers():
	reserved_features = ['Loan_ID', 'Loan_Status', 'Sample_Category']
	answer_file = "../../loan_data_selection/selected_samples.csv"
	answer_df = pd.read_csv(answer_file, usecols=reserved_features)
	answer_list = answer_df.values.tolist()
	answer_dict = {}
	for Loan_ID, Loan_Status, Sample_Category in answer_list:
		tp_answer = "accept" if Loan_Status == "Y" else "reject"
		answer_dict[Loan_ID] = tp_answer
	return answer_dict

def filter_time_threshold(threshold=360):
	filename = os.path.join(data_folder, "prolific_demographic.csv")
	usecols = ['participant_id', 'started_datetime', 'time_taken']
	df = pd.read_csv(filename, usecols=usecols)
	df = df.drop(df[df['time_taken'] < threshold].index)
	user_status_list = df.values.tolist()
	user_set = set([item[0] for item in user_status_list])
	user_round2_flag_dict = {}
	for participant_id, started_datetime, time_taken in user_status_list:
		if started_datetime.startswith("2021-08-19"):
			user_round2_flag_dict[participant_id] = False
		else:
			user_round2_flag_dict[participant_id] = True
	return user_set, user_round2_flag_dict

def filter_attention_check(user_task_dict, user_attention_check_correct, user_round2_flag_dict):
	valid_user_set = set()
	for user in user_task_dict:
		if user not in user_round2_flag_dict:
			# this user doesn't pass time filter
			continue
		if user_round2_flag_dict[user]:
			threshold = 5
		else:
			threshold = 2
		if user_attention_check_correct[user] >= threshold:
			valid_user_set.add(user)
	return valid_user_set

def collect_file_users(filename):
	df = pd.read_csv(filename, usecols=['user_id'])
	user_list = df.values.tolist()
	user_set = set([item[0] for item in user_list])
	return user_set


def check_completeness(user_task_dict, userinfo_dict):
	filenames = ["ATI_PreQ.csv", "PostQ_accuracy.csv", "PostQ_analogy.csv", "TiA_PostQ.csv", "pre_questionnaire.csv"]
	users_occ_list = []
	for filename in filenames:
		users_occ_list.append(collect_file_users(os.path.join(data_folder, filename)))
	reuqired_dict = {
		"system": [0, 3, 4],
		"accuracy": [0, 1, 3, 4],
		"analogy": [0, 2, 3, 4]
	}
	valid_user_set = set()
	for user in user_task_dict:
		if len(user_task_dict[user]) < 20 or user not in userinfo_dict:
			continue
		user_id, x, xx, user_group, analogy_type = userinfo_dict[user]
		required_checks = reuqired_dict[user_group]
		flag = False
		for tp_index in required_checks:
			if user not in users_occ_list[tp_index]:
				flag = True
				break
		if flag:
			continue
		valid_user_set.add(user)
	return valid_user_set


def filter_invalid_users(exclude_round1=False):
	valid_user_time, user_round2_flag_dict = filter_time_threshold(threshold=420)
	print("Valid user for time threshold: {}, {}".format(len(valid_user_time), len(user_round2_flag_dict)))
	user_task_dict, user_attention_check_correct = read_usertask()
	userinfo_dict = read_userinfo()
	valid_user_attention = filter_attention_check(user_task_dict, user_attention_check_correct, user_round2_flag_dict)
	print("Valid user for attention check and time: {}".format(len(valid_user_attention)))
	valid_user_completeness = check_completeness(user_task_dict, userinfo_dict)
	print("Valid user for completeness: {}".format(len(valid_user_completeness)))
	reserved_users = valid_user_attention & valid_user_completeness
	print("Valid users for time, attention check and completeness:{}".format(len(reserved_users)))
	if exclude_round1:
		round2users = set()
		for user in user_round2_flag_dict:
			if user_round2_flag_dict[user]:
				round2users.add(user)
		reserved_users = round2users & reserved_users
	group_ct = {}
	analogy_ct = {}
	for user in reserved_users:
		user_id, x, xx, user_group, analogy_type = userinfo_dict[user]
		if user_group not in group_ct:
			group_ct[user_group] = 0
		group_ct[user_group] += 1
		if user_group == "analogy":
			if analogy_type not in analogy_ct:
				analogy_ct[analogy_type] = 0
			analogy_ct[analogy_type] += 1
	for group in ["system", "accuracy", "analogy"]:
		print("Group {} has {} valid participants".format(group, group_ct[group]))
	for at in analogy_ct:
		print("Among analogy group, {} participants see {} type of analogy".format(analogy_ct[at], at))
	
	return reserved_users

def read_userinfo(reserved_users=None):
	filename = os.path.join(data_folder, "userinfo.csv")
	usecols = ['user_id','task_order_str','reverse_flag_str','user_group','analogy_type']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_status_list = df.values.tolist()
	userinfo_dict = {}
	for tuple_ in user_status_list:
		userinfo_dict[tuple_[0]] = tuple_
	return userinfo_dict

def calc_mean(value_list):
	return np.mean(value_list)

def reverse_code(value, max_scale):
	return max_scale + 1 - value

class questionnaire(object):
	
	def __init__(self, values, reverse_code_index, max_scale=6, questionnaire_size=0):
		self.questionnaire_size = questionnaire_size
		assert len(values) == self.questionnaire_size
		tp_values = []
		for i, value in enumerate(values):
			if (i + 1) in reverse_code_index:
				tp_values.append(reverse_code(value, max_scale))
			else:
				tp_values.append(value)
		self.values = tp_values
		self.max_scale = max_scale

	def calc_value(self):
		return calc_mean(self.values)

TiA_subscales = ["Reliability/Competence", "Understanding/Predictability", "Familiarity", \
				 "Intention of Developers", "Propensity to Trust", "Trust in Automation"]

class TiA_questionnaire(object):
	
	def __init__(self, values):
		self.questionnaire_size = 19
		self.max_scale = 5
		self.reverse_code_index = [5, 10, 15, 16]
		self.subscales = {
			"Reliability/Competence": [1, 6, 10, 13, 15, 19],
			"Understanding/Predictability": [2, 7, 11, 16],
			"Familiarity": [3, 17],
			"Intention of Developers": [4, 8],
			"Propensity to Trust": [5, 12, 18],
			"Trust in Automation": [9, 14]
		}
		assert len(values) == self.questionnaire_size
		tp_values = []
		for i, value in enumerate(values):
			if (i + 1) in self.reverse_code_index:
				tp_values.append(reverse_code(value, self.max_scale))
			else:
				tp_values.append(value)
		self.values = tp_values
		self.scale_dict = {}

	def calc_value(self):
		for subscale in self.subscales:
			self.scale_dict[subscale] = calc_mean([self.values[index - 1] for index in self.subscales[subscale]])
			# print(subscale, self.scale_dict[subscale])
		# print("-" * 20)
		return self.scale_dict

def calc_numeracy(reserved_users=None):
	filename = os.path.join(data_folder, "pre_questionnaire.csv")
	df = pd.read_csv(filename)
	user_numeracy_list = df.values.tolist()
	user_numeracy_dict = {}
	reverse_code_index = [7]
	max_scale = 6
	for tuple_ in user_numeracy_list:
		user_id = tuple_[0]
		# answer_7 = 7 - answer_7
		# reverse code for answer_7
		# SNS_ability_sum = answer_1 + answer_2 + answer_3 + answer_4
		# SNS_preference_sum = answer_5 + answer_6 + answer_7 + answer_8
		# SNS = (SNS_ability_sum + SNS_preference_sum) / 8.0
		# SNS_ability = SNS_ability_sum / 4.0
		# SNS_preference = SNS_preference_sum / 4.0
		SNS = questionnaire(tuple_[1:-1], reverse_code_index=[7], max_scale=max_scale, questionnaire_size=8).calc_value()
		SNS_ability = questionnaire(tuple_[1:5], reverse_code_index=[], max_scale=max_scale, questionnaire_size=4).calc_value()
		SNS_preference = questionnaire(tuple_[5:9], reverse_code_index=[3], max_scale=max_scale, questionnaire_size=4).calc_value()
		user_numeracy_dict[user_id] = (SNS, SNS_ability, SNS_preference)
		# print(user_id, SNS, SNS_ability, SNS_preference)
	return user_numeracy_dict

def calc_ATI_scale():
	filename = os.path.join(data_folder, "ATI_PreQ.csv")
	df = pd.read_csv(filename)
	user_task_list = df.values.tolist()
	user_ATI_scale = {}
	reverse_code_index = [3, 6, 8]
	max_scale = 6
	questionnaire_size = 9
	# reverse code for question 3, 6, 8
	for tuple_ in user_task_list:
		# user_id, answer_1, answer_2, answer_3, answer_4, answer_5, answer_6, answer_7, answer_8, answer_9, __ = tuple_
		# reverse code for question 3, 6, 8
		# answer_3 = 7 - answer_3
		# answer_6 = 7 - answer_6
		# answer_8 = 7 - answer_8
		# ATI_scale = (answer_1 + answer_2 + answer_3 + answer_4 + answer_5 + answer_6 + answer_7 + answer_8 + answer_9) / 9.0
		ATI_scale = questionnaire(tuple_[1:-1], reverse_code_index, max_scale=max_scale, questionnaire_size=questionnaire_size).calc_value()
		user_id = tuple_[0]
		user_ATI_scale[user_id] = ATI_scale
	return user_ATI_scale

def obtain_analogy_familiarity():
	filename = os.path.join(data_folder, "PostQ_analogy.csv")
	df = pd.read_csv(filename, usecols=['user_id', "answer_1", "answer_2"])
	analogy_feedback_list = df.values.tolist()
	user_familiarity_dict = {}
	user_usefulness_dict = {}
	for user_id, usefulness, familiarity in analogy_feedback_list:
		 user_familiarity_dict[user_id] = familiarity
		 user_usefulness_dict[user_id] = usefulness
	return user_usefulness_dict, user_familiarity_dict

def calc_TiA_scale():
	filename = os.path.join(data_folder, "TiA_PostQ.csv")
	df = pd.read_csv(filename)
	user_task_list = df.values.tolist()
	user_TiA_scale = {}
	for tuple_ in user_task_list:
		TiA_scale = TiA_questionnaire(tuple_[1:-1]).calc_value()
		user_id = tuple_[0]
		user_TiA_scale[user_id] = TiA_scale
	return user_TiA_scale


def read_usertask(reserved_users=None):
	filename = os.path.join(data_folder, "usertask.csv")
	usecols = ['user_id', 'task_id', 'answer_type', 'choice']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_task_list = df.values.tolist()

	user_task_dict = {}
	user_attention_check_correct = {}
	for user_id, task_id, answer_type, choice in user_task_list:
		if user_id not in user_task_dict:
			user_task_dict[user_id] = {}
		if user_id not in user_attention_check_correct:
			user_attention_check_correct[user_id] = 0
		if answer_type == "attention":
			if task_id == "LP001518" and choice == "yes":
				user_attention_check_correct[user_id] += 1
			if task_id == "LP002723" and choice == "no":
				user_attention_check_correct[user_id] += 1
			if task_id == "ATI_attention" and choice == "6":
				user_attention_check_correct[user_id] += 1
			if task_id == "Numeracy Test" and choice == "3":
				user_attention_check_correct[user_id] += 1
			if task_id == "TiA_attention" and choice == "1":
				user_attention_check_correct[user_id] += 1
		else:
			user_task_dict[user_id][(task_id, answer_type)] = choice
	return user_task_dict, user_attention_check_correct


def calc_reliance_measures(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	accuracy_dict = {}
	user_appropriate_reliance = {}
	user_agreement_fraction_dict = {}
	user_switching_fraction_dict = {}
	number_no_disagreement_user = 0
	for user in usertask_dict:
		user_group = user_group_dict[user]
		analogy_type = user_analogy_dict[user]
		user_trust_list = []
		tp_correct = 0
		tp_trust = 0
		tp_agreement = 0
		initial_disagreement = 0
		tp_switch_reliance = 0
		tp_correct_initial_disagreement = 0
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			if second_choice == system_advice:
				# agreement fraction
				tp_agreement += 1
			if first_choice != system_advice:
				# initial disagreement
				initial_disagreement += 1
				if second_choice == system_advice:
					tp_switch_reliance += 1
				if second_choice == correct_answer:
					tp_correct_initial_disagreement += 1
			if second_choice == correct_answer:
				tp_correct += 1
		accuracy_dict[user] = tp_correct / 10.0
		user_agreement_fraction_dict[user] = tp_agreement / 10.0
		if initial_disagreement > 0:
			user_switching_fraction_dict[user] = float(tp_switch_reliance) / initial_disagreement
			user_appropriate_reliance[user] = float(tp_correct_initial_disagreement) / initial_disagreement
		else:
			number_no_disagreement_user += 1
			user_switching_fraction_dict[user] = 0.0
			user_appropriate_reliance[user] = 0.0
	print("There are {} users agree with system advice all the time.".format(number_no_disagreement_user))
	return user_appropriate_reliance, accuracy_dict, user_agreement_fraction_dict, user_switching_fraction_dict



if __name__ == "__main__":
	# filter_invalid_users()
	# calc_ATI_scale()
	calc_TiA_scale()












